package yrpg.bamp.base.plugin.other;

import com.alibaba.fastjson.JSONObject;
import com.apusic.aas.util.codec.binary.Base64;
import kd.bos.bec.api.IEventServicePlugin;
import kd.bos.bec.model.EntityEvent;
import kd.bos.bec.model.KDBizEvent;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.exception.KDBizException;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.http.*;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import yrpg.base.common.utils.LogDataBean;
import yrpg.base.common.utils.RestRequestFactoryUtils;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 事件订阅人员保存时，同步在汇联易离职（容错增强版）
 */
public class UserEnableHlyServicePlugin implements IEventServicePlugin {
    private static final String BASE_URL = System.getProperty("yrpg.fi.em.slhly.url");
    private static final int API_TIMEOUT = 120_000;

    @Override
    public Object handleEvent(KDBizEvent evt) {
        if (evt instanceof EntityEvent) {
            EntityEvent entityEvent = (EntityEvent) evt;
            String entityNumber = entityEvent.getEntityNumber();

            for (String businessKey : entityEvent.getBusinesskeys()) {
                try {
                    processSingleRecord(businessKey, entityNumber);
                } catch (Exception e) {
                    // 单个记录处理失败不影响其他记录
                    new LogDataBean("人员处理异常", "SYSTEM", "", "", "false",
                            ExceptionUtils.getStackTrace(e)).doSaveLog();
                }
            }
        }
        return null;
    }

    /**
     * 判断状态
     * @param businessKey
     * @param entityNumber
     */
    private void processSingleRecord(String businessKey, String entityNumber) {
        DynamicObject obj = BusinessDataServiceHelper.loadSingle(businessKey, entityNumber,
                "enable,number,yrpg_isoutjob,id");
        String enable = obj.getString("enable");
        boolean isOutJob = obj.getBoolean("yrpg_isoutjob");
        String number = obj.getString("number");

        if ("0".equals(enable) && !isOutJob) {
            handleLeave(number, obj);
        } else if ("1".equals(enable) && isOutJob) {
            handleRehire(number, obj);
        }
    }

    /**
     * 员工离职
     * @param employeeId
     * @param obj
     */
    private void handleLeave(String employeeId, DynamicObject obj) {
        boolean apiSuccess = executeApiRequest(
                employeeId,
                "员工离职",
                BASE_URL + "api/open/user/delete/v2/" + employeeId + "/",
                HttpMethod.DELETE,
                buildLeaveRequestData()
        );

        if (apiSuccess) {
            updateLocalStatus(obj, true);
        }
    }

    /**
     * 员工离职恢复
     * @param employeeId
     * @param obj
     */
    private void handleRehire(String employeeId, DynamicObject obj) {
        boolean apiSuccess = executeApiRequest(
                employeeId,
                "员工离职恢复",
                BASE_URL + "api/open/user/rehire/" + employeeId + "/",
                HttpMethod.POST,
                new JSONObject()
        );

        if (apiSuccess) {
            updateLocalStatus(obj, false);
        }
    }

    /**
     * 执行API
     * @param employeeId
     * @param action
     * @param url
     * @param method
     * @param data
     * @return
     */
    private boolean executeApiRequest(String employeeId, String action, String url,
                                      HttpMethod method, JSONObject data) {
        try {
            String token = login();
            HttpHeaders headers = buildHeaders(token);

            LogDataBean log = new LogDataBean(
                    action, "汇联易", url, data.toJSONString(), "true", "");

            String response = RestRequestFactoryUtils.doBodyRequest(
                    API_TIMEOUT, headers, data, url, method);

            return handleApiResponse(response, log);
        } catch (Exception ex) {
            logError(action, url, ex);
            return false;
        }
    }

    /**
     * 处理返回值
     * @param response
     * @param log
     * @return
     */
    private boolean handleApiResponse(String response, LogDataBean log) {
        try {
            JSONObject responseJson = JSONObject.parseObject(response);
            if ("0000".equals(responseJson.getString("errorCode"))) {
                log.setStatus("true");
                log.setResponse(response);
                log.doSaveLog();
                return true;
            } else {
                log.setStatus("false");
                log.setResponse(responseJson.getString("message"));
                log.doSaveLog();
                return false;
            }
        } catch (Exception e) {
            log.setStatus("false");
            log.setResponse("响应解析失败: " + ExceptionUtils.getStackTrace(e));
            log.doSaveLog();
            return false;
        }
    }

    /**
     * 错误处理
     * @param action
     * @param url
     * @param ex
     */
    private void logError(String action, String url, Exception ex) {
        new LogDataBean(action, "汇联易", url, "", "false",
                "ERROR: " + ExceptionUtils.getStackTrace(ex))
                .doSaveLog();
    }

    /**
     * 离职请求参数
     * @return
     */
    private JSONObject buildLeaveRequestData() {
        JSONObject data = new JSONObject();
        data.put("leavingDate", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        return data;
    }

    /**
     * 拼接请求头
     * @param token
     * @return
     */
    private HttpHeaders buildHeaders(String token) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("Authorization", "Bearer " + token);
        return headers;
    }

    /**
     * 更新人员状态
     * @param obj
     * @param isOutJob
     */
    private void updateLocalStatus(DynamicObject obj, boolean isOutJob) {
        try {
            obj.set("yrpg_isoutjob", isOutJob);
            SaveServiceHelper.update(obj);
        } catch (Exception e) {
            new LogDataBean("人员同步后本地状态更新", "SYSTEM", "", "", "false",
                    "状态更新失败: " + ExceptionUtils.getStackTrace(e))
                    .doSaveLog();
        }
    }

    /**
     * 登录调用
     * @return
     */
    private String login() {
        try {
            RestTemplate restTemplate = createRestTemplate();
            HttpEntity<MultiValueMap<String, String>> entity = buildLoginEntity();

            ResponseEntity<String> response = restTemplate.exchange(
                    BASE_URL + "oauth/token",
                    HttpMethod.POST,
                    entity,
                    String.class
            );

            return JSONObject.parseObject(response.getBody()).getString("access_token");
        } catch (Exception ex) {
            throw new KDBizException("登录失败: " + ex.getMessage());
        }
    }

    private RestTemplate createRestTemplate() {
        HttpComponentsClientHttpRequestFactory factory =
                new HttpComponentsClientHttpRequestFactory();
        factory.setConnectTimeout(API_TIMEOUT);
        factory.setReadTimeout(API_TIMEOUT);
        return new RestTemplate(factory);
    }

    private HttpEntity<MultiValueMap<String, String>> buildLoginEntity() {
        String credentials = System.getProperty("yrpg.fi.em.slhly.url.clientId") + ":" +
                System.getProperty("yrpg.fi.em.slhly.url.securet");
        String authHeader = "Basic " + Base64.encodeBase64String(credentials.getBytes());

        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", authHeader);

        MultiValueMap<String, String> body = new LinkedMultiValueMap<>();
        body.add("grant_type", "client_credentials");
        body.add("scope", "write");

        return new HttpEntity<>(body, headers);
    }
}