package yrpg.bamp.base.plugin.task;

import com.alibaba.fastjson.JSONObject;
import com.apusic.aas.util.codec.binary.Base64;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.exception.KDBizException;
import kd.bos.exception.KDException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.http.*;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import yrpg.base.common.utils.LogDataBean;
import yrpg.base.common.utils.RestRequestFactoryUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * 离职人员同步汇联易定时任务
 */
public class UserEnableHlyTask extends AbstractTask implements Plugin {
    private static final String BASE_URL = System.getProperty("yrpg.fi.em.slhly.url");
    private static final int API_TIMEOUT = 120_000;
    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        //离职但是却没有同步汇联易离职的
        QFilter filter1 = new QFilter("enable", QCP.equals, "0").and(new QFilter("yrpg_isoutjob", QCP.not_equals, true));
        //在职但是却在汇联易离职的
        QFilter filter2 = new QFilter("enable", QCP.equals, "1").and(new QFilter("yrpg_isoutjob", QCP.equals, true));
        QFilter or = filter1.or(filter2);
        QFilter baseFilter = new QFilter("masteruserid", QCP.is_notnull, "").and(new QFilter("masteruserid", QCP.not_equals, "")).and(new QFilter("idcard", QCP.is_notnull, "").and(new QFilter("idcard", QCP.not_equals, "")));
        QFilter filter = or.and(baseFilter);
        //查询系统中已经禁用，但是却没有同步汇联易离职状态的人员
        DynamicObject[] users = BusinessDataServiceHelper.load("bos_user", "enable,number,yrpg_isoutjob,id", filter.toArray());
        for (DynamicObject user : users) {
            try{
                processSingleRecord(user);
            }catch (Exception e) {
                // 单个记录处理失败不影响其他记录
                new LogDataBean("人员处理异常", "SYSTEM", "", "", "false",
                        ExceptionUtils.getStackTrace(e)).doSaveLog();
            }
        }

    }
    /**
     * 判断状态
     */
    private void processSingleRecord(DynamicObject user) {
        String enable = user.getString("enable");
        boolean isOutJob = user.getBoolean("yrpg_isoutjob");
        String number = user.getString("number");

        if ("0".equals(enable) && !isOutJob) {
            handleLeave(number, user);
        } else if ("1".equals(enable) && isOutJob) {
            handleRehire(number, user);
        }
    }

    /**
     * 员工离职
     * @param employeeId
     * @param obj
     */
    private void handleLeave(String employeeId, DynamicObject obj) {
        boolean apiSuccess = executeApiRequest(
                employeeId,
                "员工离职",
                BASE_URL + "api/open/user/delete/v2/" + employeeId + "/",
                HttpMethod.DELETE,
                buildLeaveRequestData()
        );

        if (apiSuccess) {
            updateLocalStatus(obj, true);
        }
    }

    /**
     * 员工离职恢复
     * @param employeeId
     * @param obj
     */
    private void handleRehire(String employeeId, DynamicObject obj) {
        boolean apiSuccess = executeApiRequest(
                employeeId,
                "员工离职恢复",
                BASE_URL + "api/open/user/rehire/" + employeeId + "/",
                HttpMethod.POST,
                new JSONObject()
        );

        if (apiSuccess) {
            updateLocalStatus(obj, false);
        }
    }

    /**
     * 执行API
     * @param employeeId
     * @param action
     * @param url
     * @param method
     * @param data
     * @return
     */
    private boolean executeApiRequest(String employeeId, String action, String url,
                                      HttpMethod method, JSONObject data) {
        try {
            String token = login();
            HttpHeaders headers = buildHeaders(token);

            LogDataBean log = new LogDataBean(
                    action, "汇联易", url, data.toJSONString(), "true", "");

            String response = RestRequestFactoryUtils.doBodyRequest(
                    API_TIMEOUT, headers, data, url, method);

            return handleApiResponse(response, log);
        } catch (Exception ex) {
            logError(action, url, ex);
            return false;
        }
    }

    /**
     * 处理返回值
     * @param response
     * @param log
     * @return
     */
    private boolean handleApiResponse(String response, LogDataBean log) {
        try {
            JSONObject responseJson = JSONObject.parseObject(response);
            if ("0000".equals(responseJson.getString("errorCode"))) {
                log.setStatus("true");
                log.setResponse(response);
                log.doSaveLog();
                return true;
            } else {
                log.setStatus("false");
                log.setResponse(responseJson.getString("message"));
                log.doSaveLog();
                return false;
            }
        } catch (Exception e) {
            log.setStatus("false");
            log.setResponse("响应解析失败: " + ExceptionUtils.getStackTrace(e));
            log.doSaveLog();
            return false;
        }
    }

    /**
     * 错误处理
     * @param action
     * @param url
     * @param ex
     */
    private void logError(String action, String url, Exception ex) {
        new LogDataBean(action, "汇联易", url, "", "false",
                "ERROR: " + ExceptionUtils.getStackTrace(ex))
                .doSaveLog();
    }

    /**
     * 离职请求参数
     * @return
     */
    private JSONObject buildLeaveRequestData() {
        JSONObject data = new JSONObject();
        data.put("leavingDate", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        return data;
    }

    /**
     * 拼接请求头
     * @param token
     * @return
     */
    private HttpHeaders buildHeaders(String token) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("Authorization", "Bearer " + token);
        return headers;
    }

    /**
     * 更新人员状态
     * @param obj
     * @param isOutJob
     */
    private void updateLocalStatus(DynamicObject obj, boolean isOutJob) {
        try {
            obj.set("yrpg_isoutjob", isOutJob);
            SaveServiceHelper.update(obj);
        } catch (Exception e) {
            new LogDataBean("人员同步后本地状态更新", "SYSTEM", "", "", "false",
                    "状态更新失败: " + ExceptionUtils.getStackTrace(e))
                    .doSaveLog();
        }
    }

    /**
     * 登录调用
     * @return
     */
    private String login() {
        try {
            RestTemplate restTemplate = createRestTemplate();
            HttpEntity<MultiValueMap<String, String>> entity = buildLoginEntity();

            ResponseEntity<String> response = restTemplate.exchange(
                    BASE_URL + "oauth/token",
                    HttpMethod.POST,
                    entity,
                    String.class
            );

            return JSONObject.parseObject(response.getBody()).getString("access_token");
        } catch (Exception ex) {
            throw new KDBizException("登录失败: " + ex.getMessage());
        }
    }

    private RestTemplate createRestTemplate() {
        HttpComponentsClientHttpRequestFactory factory =
                new HttpComponentsClientHttpRequestFactory();
        factory.setConnectTimeout(API_TIMEOUT);
        factory.setReadTimeout(API_TIMEOUT);
        return new RestTemplate(factory);
    }

    private HttpEntity<MultiValueMap<String, String>> buildLoginEntity() {
        String credentials = System.getProperty("yrpg.fi.em.slhly.url.clientId") + ":" +
                System.getProperty("yrpg.fi.em.slhly.url.securet");
        String authHeader = "Basic " + Base64.encodeBase64String(credentials.getBytes());

        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", authHeader);

        MultiValueMap<String, String> body = new LinkedMultiValueMap<>();
        body.add("grant_type", "client_credentials");
        body.add("scope", "write");

        return new HttpEntity<>(body, headers);
    }
}
